*************************************************************
*** Analyze Monte Carlo experiments (SLX data-generating process with multiple Ws) for the Drolc, Gandrud and Williams PSJ project
***
*** Created: 5-2-19
*** Modified: 
***
*************************************************************

clear 
clear matrix
clear mata
set matsize 1000
set maxvar 32000

version 14

use "Experiments\Multiple Ws\Data\SLX with Multiple Ws.dta", clear

local N = 1000 			/* Number of simulations */

*********************************** Bias in Coefficients **********************************

*** Mean Absolute Bias (MAB) for the coefficients
qui foreach v of varlist b_x_m1 b_x_m2 b_x_m3 {
	bys corr_12 corr_13: egen mab_`v' = sum(abs(`v' - 1)/`N')
}

qui foreach v of varlist theta_1_m1 theta_1_m2 {
	bys corr_12 corr_13: egen mab_`v' = sum(abs(`v' - 0.1)/`N')
}

qui foreach v of varlist theta_2_m1 {
	bys corr_12 corr_13: egen mab_`v' = sum(abs(`v' - 0.25)/`N')
}

qui foreach v of varlist theta_3_m3 {
	bys corr_12 corr_13: egen mab_`v' = sum(abs(`v' - 0)/`N')
}

*********************************** Bias AND efficiency in Coefficients **********************************

*** RMSE for the coefficients
qui foreach v of varlist b_x_m1 b_x_m2 b_x_m3 {
	bys corr_12 corr_13: egen mse_`v' = sum((`v' - 1)^2/`N')
	replace mse_`v' = sqrt(mse_`v')
}

qui foreach v of varlist theta_1_m1 theta_1_m2 {
	bys corr_12 corr_13: egen mse_`v' = sum((`v' - 0.1)^2/`N')
	replace mse_`v' = sqrt(mse_`v')
	
	bys corr_12 corr_13: egen mn_`v' = mean(`v')
}

qui foreach v of varlist theta_2_m1 {
	bys corr_12 corr_13: egen mse_`v' = sum((`v' - 0.25)^2/`N')
	replace mse_`v' = sqrt(mse_`v')
}

qui foreach v of varlist theta_3_m3 {
	bys corr_12 corr_13: egen mse_`v' = sum((`v' - 0)^2/`N')
	replace mse_`v' = sqrt(mse_`v')
}

*********************************** Bias in Standard Errors **********************************

*** 90% coverage probabilities
qui foreach v in b_x {
	foreach m of numlist 1(1)3 {
		bys corr_12 corr_13: gen overlap_`v'_m`m' = cond((`v'_m`m' - 1.65 * se_`v'_m`m' < true_`v' & `v'_m`m' + 1.65 * se_`v'_m`m' > true_`v'), 1, 0)
		bys corr_12 corr_13: egen cover_`v'_m`m' = mean(overlap_`v'_m`m')
		drop overlap_`v'_m`m'
	}
}

qui foreach v in theta_1 {
	foreach m of numlist 1 2 {
		bys corr_12 corr_13: gen overlap_`v'_m`m' = cond((`v'_m`m' - 1.65 * se_`v'_m`m' < true_`v' & `v'_m`m' + 1.65 * se_`v'_m`m' > true_`v'), 1, 0)
		bys corr_12 corr_13: egen cover_`v'_m`m' = mean(overlap_`v'_m`m')
		drop overlap_`v'_m`m'
	}
}

qui foreach v in theta_2 {
	foreach m of numlist 1 {
		bys corr_12 corr_13: gen overlap_`v'_m`m' = cond((`v'_m`m' - 1.65 * se_`v'_m`m' < true_`v' & `v'_m`m' + 1.65 * se_`v'_m`m' > true_`v'), 1, 0)
		bys corr_12 corr_13: egen cover_`v'_m`m' = mean(overlap_`v'_m`m')
		drop overlap_`v'_m`m'
	}
}

qui foreach v in theta_3 {
	foreach m of numlist 3 {
		bys corr_12 corr_13: gen overlap_`v'_m`m' = cond((`v'_m`m' - 1.65 * se_`v'_m`m' < true_`v' & `v'_m`m' + 1.65 * se_`v'_m`m' > true_`v'), 1, 0)
		bys corr_12 corr_13: egen cover_`v'_m`m' = mean(overlap_`v'_m`m')
		drop overlap_`v'_m`m'
	}
}


*********************************** Bias in Effects **********************************

*** Mean Absolute Bias (MAB)
qui foreach m of numlist 1(1)3 {
	foreach e in t d i {
		bys corr_12 corr_13: egen mab_`e'effect_m`m' = sum(abs(`e'effect_m`m' - true_`e'effect)/`N')
	}
}

*** RMSE
qui foreach m of numlist 1(1)3 {
	foreach e in t d i {
		bys corr_12 corr_13: egen mse_`e'effect_m`m' = sum((`e'effect_m`m' - true_`e'effect)^2/`N')
		replace mse_`e'effect_m`m' = sqrt(mse_`e'effect_m`m')
	}
}

*** Falsely reject true null that \theta_3 == 0
gen reject = cond((theta_3_m3 - (1.65 * se_theta_3_m3) > 0) | (theta_3_m3 + (1.65 * se_theta_3_m3)) < 0, 1, 0) 
bys corr_12 corr_13: egen reject_theta_3_m3 = mean(100*reject)
drop reject

*** Generate data for box-whisker plots
bys corr_12: egen median_theta_1_m2 = pctile(theta_1_m2) if corr_13 == 0, p(50)
bys corr_12: egen lo_theta_1_m2 = pctile(theta_1_m2) if corr_13 == 0, p(5)
bys corr_12: egen hi_theta_1_m2 = pctile(theta_1_m2) if corr_13 == 0, p(95)

duplicates drop corr_12 corr_13, force

*** Bias (MAB) in coefficients
list corr_12 corr_13 mab_b_x_m1 mab_b_x_m2 mab_b_x_m3
list corr_12 corr_13 mab_theta_1_m1 mab_theta_1_m2 mab_theta_2_m1 mab_theta_3_m3

*** Bias and efficiency (MSE) in coefficients
list corr_12 corr_13 mse_b_x_m1 mse_b_x_m2 mse_b_x_m3
list corr_12 corr_13 mse_theta_1_m1 mse_theta_1_m2 mse_theta_2_m1 mse_theta_3_m3

*** Bias (coverage probability) in standard errors
list corr_12 corr_13 cover_b_x_m1 - cover_b_x_m3
list corr_12 corr_13 cover_theta_1_m1 cover_theta_1_m2 cover_theta_2_m1 cover_theta_3_m3
list corr_12 corr_13 reject_theta_3_m3

*** Bias (MAB) in effects
list corr_12 corr_13 mab_teffect*
list corr_12 corr_13 mab_deffect*
list corr_12 corr_13 mab_ieffect*

*** Bias and efficiency (MSE) in effects
list corr_12 corr_13 mse_teffect*
list corr_12 corr_13 mse_deffect*
list corr_12 corr_13 mse_ieffect*

drop sim time true_* deffect_* ieffect_* teffect_* se_* theta_* mn_theta_1_m1

reshape long aux_x1_m aux_x2_m b_x_m cover_b_x_m cover_theta_1_m cover_theta_2_m cover_theta_3_m deffect_m mab_b_x_m mab_deffect_m mab_ieffect_m mab_teffect_m mab_theta_1_m mab_theta_2_m mab_theta_3 mse_b_x_m mse_deffect_m mse_ieffect_m mse_teffect_m mse_theta_1_m mse_theta_2_m mse_theta_3_m reject_theta_3_m median_theta_1_m lo_theta_1_m hi_theta_1_m, i(corr_12 corr_13) j(model)

gen modelname = "Model 1" if model == 1
replace modelname = "Model 2" if model == 2
replace modelname = "Model 3" if model == 3

saveold "Figures\Data\SLX.dta", version(12) replace
