*************************************************************
*** Do some Monte Carlo experiments (SLX data-generating process with multiple Ws) for the Drolc, Gandrud and Williams PSJ project
***
*** Created: 4-30-19
*** Modified: 9-5-19
***
*************************************************************

clear 
clear matrix
clear mata
set matsize 10000
set maxvar 32000

version 14

do "Programs\SLX Programs.do"

*************************************************************
*** SLX with multiple sets of connections
*************************************************************

*************************************************************
*** Set up the parameters of the MC experiments
*************************************************************
local beta = 1				/* Parameters for beta_x */
local N = 500				/* Sample size & dimensions for W */

local s = 2					/* Standard deviation of the errors */
local MCsims = 1000			/* Number of simulations */
local QIsims = 1			/* Number of effects draws: necessary if calculating SEs of effects */
local obs = `N'^2			/* Number of observations */
local seed = 648			/* Set the seed */

set seed `seed'
set obs `obs'
if `N' > `QIsims' {
	set matsize `N'
}
else {
	set matsize `QIsims'
}
*************************************************************

*************************************************************
*** Generate the weights matrices with varying network correlations
*************************************************************

*** Correlation coefficients for W_1/W_2 and W_1/W_3
mat corr = (-.6, -.4, -.2, 0, .2, .4, .6)
local end = colsof(corr)

*** Coefficients for theta
local theta_1 = 0.1
local theta_2 = 0.25

*************************************************************
*** Monte Carlo Experiments
*************************************************************
tempname slx
postfile `slx' sim str30 time corr_12 corr_13 /*
*/	true_b_x true_theta_1 true_theta_2 true_theta_3 /*
*/	true_teffect true_deffect true_ieffect /*
*/	aux_x2_m2 aux_x1_m3 aux_x2_m3 /*
*/	b_x_m1 se_b_x_m1 theta_1_m1 se_theta_1_m1 theta_2_m1 se_theta_2_m1 /*
*/	teffect_m1 deffect_m1 ieffect_m1 /*
*/	b_x_m2 se_b_x_m2 theta_1_m2 se_theta_1_m2 /*
*/	teffect_m2 deffect_m2 ieffect_m2 /*
*/	b_x_m3 se_b_x_m3 theta_3_m3 se_theta_3_m3 /*
*/	teffect_m3 deffect_m3 ieffect_m3 /*
*/	using "Data\SLX with Multiple Ws.dta", replace every(10)

local stfull = "$S_TIME"
local operfull = "SLX DGP: `MCsims' simulations of `end' scenarios"


qui foreach corr12 of numlist 1(1)`end' {
	foreach corr13 of numlist 1(1)`end' {
		
		local r12 = corr[1,`corr12']
		local r13 = corr[1,`corr13']
	
		nois display _newline(1) "$S_DATE : `c(current_time)': Scenario [`r12'; `r13']"
		
		mat CORR = (1, `r12', `r13' \ `r12', 1, 0.1 \ `r13', 0.1, 1)

		drawnorm W1_ W2_ W3_, corr(CORR) n(`obs')
		egen col = fill(1(1)`N' 1(1)`N' 1(1)`N' 1(1)`N')
		sort col
		egen row = fill(1(1)`N' 1(1)`N' 1(1)`N' 1(1)`N')

		foreach w of numlist 1(1)3 {
			replace W`w'_ = 0 if row == col
		}

		reshape wide W1_ W2_ W3_, i(row) j(col)
		order W1_* W2_* W3_*

		foreach w of numlist 1(1)3 {
			mkmat W`w'_*, matrix(W`w')
		}

		*** True effects
		scalar beta = `beta'
		scalar theta_1 = `theta_1'
		scalar theta_2 = `theta_2'
		scalar theta_3 = 0
		
		matrix true = [`beta', `theta_1', `theta_2']
		mata: true = st_matrix("true")
		mata: W1 = st_matrix("W1")
		mata: W2 = st_matrix("W2")
		mata: W3 = st_matrix("W3")
		mata: calceffslx2(true, W1, W2, 1)
		
		foreach e in t d i {
			scalar true_`e'effect = r(`e'effect)
		}
		
		foreach i of numlist 1(1)`MCsims' {
			local st1 = "$S_TIME"
			local oper1 = "Generate the variables and estimate SLX models"
			
			drawnorm r, n(`N') sd(`s') clear
			generate x = uniform()

			*** Generate the three spatial-lags (based on the same X)
			mkmat x
			foreach m of numlist 1(1)3 {
				mat slag`m' = W`m' * x
				svmat slag`m'
				rename slag`m'1 slag`m'
			}

			*=========================================================
			*=========================================================
			*** Generate the SLX data-generating process
			gen y = (`beta' * x) + (`theta_1' * slag1) + (`theta_2' * slag2) + r

			*=========================================================
			*=========================================================
			
			if `obs' > `QIsims' {
				set obs `obs'
			}
			else {
				set obs `QIsims'
			}	
	
			* Model 1: Correct model
			local m = 1
			
			reg y x slag1 slag2
			mat V = e(V)
			scalar b_x_m`m' = _b[x]
			scalar se_x_m`m' = _se[x]
			
			foreach t of numlist 1 2 {
				scalar theta_`t'_m`m' = _b[slag`t']
				scalar se_theta_`t'_m`m' = _se[slag`t']
			}

			* Effects
			matrix draws = [_b[x], _b[slag1], _b[slag2]]
			mata: draws = st_matrix("draws")
			mata: calceffslx2(draws, W1, W2, `QIsims')
			
			foreach e in t d i {
				scalar `e'effect_m`m' = r(`e'effect)
			}			
				
			* Model 2: Underfitted model: W1 only
			local m = 2
			
			reg y x slag1
			mat V = e(V)
			scalar b_x_m`m' = _b[x]
			scalar se_x_m`m' = _se[x]
			
			foreach t of numlist 1 {
				scalar theta_`t'_m`m' = _b[slag`t']
				scalar se_theta_`t'_m`m' = _se[slag`t']
			}			
			
			* Effects
			matrix draws = [_b[x], _b[slag1]]
			mata: draws = st_matrix("draws")
			mata: calceffslx1(draws, W1, `QIsims')
			
			foreach e in t d i {
				scalar `e'effect_m`m' = r(`e'effect)
			}						
			
			* Auxiliary regression: effect of W_1 X on W2 X
			reg slag2 slag1
			scalar aux_x2_m`m' = _b[slag1]
			
			* Model 3: Misspecified model: W3
			local m = 3
			
			reg y x slag3
			mat V = e(V)
			scalar b_x_m`m' = _b[x]
			scalar se_x_m`m' = _se[x]
			
			foreach t of numlist 3 {
				scalar theta_`t'_m`m' = _b[slag`t']
				scalar se_theta_`t'_m`m' = _se[slag`t']
			}			
			
			* Effects
			matrix draws = [_b[x], _b[slag3]]
			mata: draws = st_matrix("draws")
			mata: calceffslx1(draws, W3, `QIsims')
			
			foreach e in t d i {
				scalar `e'effect_m`m' = r(`e'effect)
			}								
			
			* Auxiliary regression: effect of W_3 X on W_2 X AND W_3 X on W_1 X
			reg slag1 slag3
			scalar aux_x1_m`m' = _b[slag3]
			
			reg slag2 slag3
			scalar aux_x2_m`m' = _b[slag3]			
			
			if mod(`i',5) == 0 {
				nois display "." _c
				if mod(`i',250) == 0 {
					nois display "$S_DATE : `c(current_time)': Simulation #`i'"
				}
			}		
		
		elapse "`st1'" "`oper1'"
		
		post `slx' (`i') ("$S_elap") (`r12') (`r13') /*
		*/	(beta) (theta_1) (theta_2) (theta_3) /*		
		*/	(true_teffect) (true_deffect) (true_ieffect) /*	
		*/	(aux_x2_m2) (aux_x1_m3) (aux_x2_m3) /*
		*/	(b_x_m1) (se_x_m1) (theta_1_m1) (se_theta_1_m1) (theta_2_m1) (se_theta_2_m1) /*
		*/	(teffect_m1) (deffect_m1) (ieffect_m1) /*
		*/	(b_x_m2) (se_x_m2) (theta_1_m2) (se_theta_1_m2) /*
		*/	(teffect_m2) (deffect_m2) (ieffect_m2) /*
		*/	(b_x_m3) (se_x_m3) (theta_3_m3) (se_theta_3_m3)	/*
		*/	(teffect_m3) (deffect_m3) (ieffect_m3) 
		
		keep in 1/`obs'	
		drop r x y slag* 
		}
	}
}

postclose `slx'
elapse "`stfull'" "`operfull'"
